/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 6.8 (Heavy Tailed Simple RoU, p. 151)
 *
 * this file: algo06_08.c
 *
 * For a demo how to use this algorithm see the file "demo_algo06_08.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

#define POW(a,b) exp(log(a)*b)

static double ul, ur, vm, m, r, (*pdfh)(double);

int setup06_08( double (*pdf)(double), double Af, double mi, double Fm, double ri)
/*
 * Algorithm 6.8 (Heavy Tailed Simple RoU, p. 151)
 *
 * pdf ... function pointer to the quasi density; must be heavy-tailed T_c conave!!
           with c <= -1/2
 * Af ... area below the quasi density
 * mi ... mode of the density
 * Fm ... CDF value at the mode m; if unknown -1. must be used
 * ri ... parameter of the distribution: we must have r >= 1 and
 *       for the c of heavy tailed T-c concave  we must take r = -c/(1+c)
 */
{     double um;
      pdfh = pdf;
      m = mi;
      r = ri;

/*1*/ vm = POW((*pdfh)(m),1./(r+1.));
      um = Af/(r*vm);
/*2*/ if(Fm >= 0.){/* the cdf value at the mode is known */
/*3*/    ul = - Fm*um;
         ur = um+ul;
      }
/*4*/ else{
/*5*/    ul = -um;
         ur = um;
      }
      printf("Setup06_08:\n ul %f ur %f vm %f\n\n",ul,ur,vm);
      return 1;
} 

double generator06_08()
/* is called after the setup to generate random variates */
{ double u,v,x;

/*6*/ while(1){
/*7*/   u = ul+(ur-ul)*uniform();
        v = vm*uniform();
/*8*/   x = u/POW(v,r)+m;
/*9*/   if(POW(v,(r+1.))<=(*pdfh)(x))
/*10*/    return x;
      }
}


