/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 9.4 (Hazard Rate Constant Thinning), p. 197 
 *
 * this file: algo09_04.c
 *
 * For a demo how to use this algorithm see the file "demo_algo09_04.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

double generator09_04(double (*h)(double), double M)
/*
 * Algorithm 9.4 (Hazard Rate Constant Thinning), p. 197 
 *
 * h   ... function pointer to the bounded hazard rate h(x)
 * M   ... upper bound of the hazard rate h(x)
 */
{     double x, u, v, e, lamda;
      int i;  

/*1*/ x = 0.;
      i = 1;
      lamda = M;
/*2*/ while(1){
/*3*/	u = uniform();
/*4*/	e = -log(1.-u)/lamda;
/*5*/	x = x + e;
/*6*/	v = lamda*uniform();
/*7*/	if(v <= (*h)(x)) 
/*8*/     return x;
	i++;
      }
}



