/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag
 * 
 * Code for Algorithm 10.1 (Indexed-Search-Tail), p. 217
 *
 * this file: algo10_01.c
 *
 * For a demo how to use this algorithm see the file "demo_algo10_01.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

  static int *gt=NULL,Ch;
  static double *pc=NULL, (*pmfh)(int);


int setup10_01(double (*pmf)(int), int C)
/*
 * Algorithm 10.1 (Indexed-Search-Tail), p. 217
 * generates discrete variates
 *
 * pmf... function pointer to probability (mass) function for k >= 0
 * C ... length of guide table
 *
 */
{ double p, psum=0.;
 int j,k,l;

      if(pc!=NULL){
	free(pc);
	free(gt);
      }
      l = C;
      pc = malloc(l*sizeof(double));
      gt = malloc(C*sizeof(int));
      Ch = C;
      pmfh = pmf;
/*1*/ k = -1;
/*2*/ do{
/*3*/	k++;
        if(k==l){
	  l*=2;
	  pc=realloc(pc,l*sizeof(double));
	} 
/*4*/	p = (*pmf)(k);
	psum += p;
	pc[k]=psum;
/*5*/ }while(pc[k]<1.-1./C);
      l = k;
      pc=realloc(pc,(l+1)*sizeof(double));
/*6*/ gt[0]=0;  
      k=0;
/*7*/ for(j=1; j<C; j++){
/*8*/	while(j/(double)C > pc[k]) 
/*9*/	  k++;
/*10*/	gt[j] = k;
      }
      printf("setup10_01:\n last guidetable entry %d with CDF = %f\n\n",k,pc[k]);
      return 1;
}

int generator10_01()
/* is called after the setup to generate random variates */
{      double u,s;
       int x,j;
/*11*/ u = uniform();
/*12*/ j = (int)(u*Ch);
/*13*/ x = gt[j];
/*14*/ if(j< Ch-1){
/*15*/	 while(u > pc[x]) 
/*16*/     x++;
/*17*/	 return x;
       }
/*18*/ else{
/*19*/	 s = pc[x];
/*20*/	 while(u > s){
/*21*/	   x++;
/*22*/	   s =s + (*pmfh)(x);
	 }
/*23*/	 return x;
       }
}

int free10_01()
/* to free the allocated memory*/
{
    free(pc);
    pc = NULL;
    free(gt);
    gt = NULL;
    return 1;
}



