/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 2.9 (Ratio-of-uniforms), p. 35 
 *
 * this file: demo_algo02_09.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo02_09 demo_algo02_09.c algo02_09.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"

/* PDF of the distribution
   you can insert your own distribution here
*/
double my_pdf(double x)
/* PDF of gamma distribution with shape parameter alpha = 3 */
{ 
  return (x<=0. ? 0. : exp(-x)*x*x);
}




#define CLAS 500 /*sets number of classes for the chi-square test*/

static long int obs[CLAS*CLAS];

int main()
/* demonstrates the use of the algorithm
   and conducts a chi-square test */
{ int i;
 double x;

 setseed(133248); /*set seed for uniform generator*/

 printf("Demo for Algorithm 2.9\nfirst we display ten numbers generated from my_pdf()\n");
 for(i=0;i<10;i++){
   x = generator02_09(my_pdf, 2.,-0.6449 ,1.23602 ,0.73576 );
   printf("x %f\n",x);
 }
 
 printf("\nNow we make a chi-square test with sample size %d for the Gamma(3) distribution:\n\n", CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;

 for(i=0;i<CLAS*CLAS;i++){
   x = generator02_09(my_pdf, 2.,-0.6449 ,1.23602 ,0.73576 );
   obs[(int)(cdf_gamma3(x)*CLAS)]++;
 }

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}

