/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 5.3 (Ahrens, equal area), p. 122  
 *
 * this file: demo_algo05_03.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo05_03 demo_algo05_03.c algo05_03.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"


/* PDF of the distribution and its derivative (dpdf)
   you can insert your own distribution here
*/

double my_pdf(double x)
{return 2.-2.*x;}
/* triangular on (0,1) */

double my_dpdf(double x)
/* derivative of the triangular pdf on (0,1) */
{ 
  if( x >= 0. && x <= 1.)
    return -2.;
  else return 0.;
}
double my_ddpdf(double x)
/* derivative of the triangular pdf on (0,1) */
{ 
  return 0.;
}




#define CLAS 500 /*sets number of classes for the chi-square test*/

static long int obs[CLAS];

int main()
/* demonstrates the use of the algorithm
   and conducts a chi-square test */
{ int i;
 double x;

 setseed(13324821); /*set seed for uniform generator*/

 printf("Demo for Algorithm 5.3\nfirst we display ten numbers generated from my_pdf()\n");
 setup05_03(my_pdf,1.,10,0.,1.);/* calls set-up*/
 for(i=0;i<10;i++){
   x = generator05_03();
   printf("x %f\n",x);
 }
 free05_03();/* to clear the memory */

 printf("\nChi-square test with sample size %d for the half-normal distribution\n\n", CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;
 setup05_03(pdf_normal,0.5,20,0.,6.);/* calls set-up*/
 for(i=0;i<CLAS*CLAS;i++){
   x = generator05_03();
   obs[(int)(cdf_halfnorm(x)*CLAS)]++;
 }
 free05_03();/* to clear the memory */

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}

