/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 6.2 (Monotone-bounded, p. 128)
 *
 * this file: demo_algo06_02.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo06_02 demo_algo06_02.c algo06_02.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"


/* PDF of the distribution and its derivative (dpdf)
   you can insert your own distribution here
*/

double my_pdf(double x)
{return 4.-x;}
/* triangular on (2,4) */



#define CLAS 500 /*sets number of classes for the chi-square test*/

static long int obs[CLAS];

int main()
/* demonstrates the use of the algorithm
   and conducts a chi-square test */
{ int i;
 double x;

 setseed(13324821); /*set seed for uniform generator*/ 

 printf("Demo for Algorithm 6.2\nfirst we display ten numbers generated from my_pdf()\n");
 setup06_02(my_pdf,2.,2.,4.);/* calls set-up*/
 for(i=0;i<10;i++){
   x = generator06_02();
   printf("x %f\n",x);
 }

 printf("\nChi-square test with sample size %d for the half-normal distribution\n\n", CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;
 setup06_02(pdf_normal,0.5,0.,6.);/* calls set-up*/
 for(i=0;i<CLAS*CLAS;i++){
   x = generator06_02();
   obs[(int)(cdf_halfnorm(x)*CLAS)]++;
 }

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}

