/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 6.3 (Monotone-unbounded, p. 130)
 *
 * this file: demo_algo06_03.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo06_03 demo_algo06_03.c algo06_03.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"


/* PDF of the distribution and its derivative (dpdf)
   you can insert your own distribution here
*/

double my_pdf(double x)
{return 2.-x;}
/* triangular on (0,1) */
/* mu is 1/3 thus mu_1 = 1/3 */


#define CLAS 500 /*sets number of classes for the chi-square test*/

static long int obs[CLAS];

int main()
/* demonstrates the use of the algorithm
   and conducts a chi-square test */
{ int i;
 double x;

 setseed(13324821); /*set seed for uniform generator*/ 

 printf("Demo for Algorithm 6.3\nfirst we display ten numbers generated from my_pdf()\n");
 setup06_03(my_pdf,1.,1./3.);/* calls set-up*/
 for(i=0;i<10;i++){
   x = generator06_03();
   printf("x %f\n",x);
 }

 printf("\nChi-square test with sample size %d for the half-normal distribution\n\n", CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;
 setup06_03(pdf_halfnorm,2.,1./0.64);/* calls set-up*/
 for(i=0;i<CLAS*CLAS;i++){
   x = generator06_03();
   obs[(int)(cdf_halfnorm(x)*CLAS)]++;
 }

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}

