/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 7.1 (Fast-Table-Inversion, p. 163)
 *
 * this file: demo_algo07_01.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo07_01 demo_algo07_01.c algo07_01.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"


/* PDF of the distribution and its derivative (dpdf)
   you can insert your own distribution here
*/

double my_pdf(double x)
/* pdf of exponential distribution */
{return exp(-x);}

double my_cdf(double x)
/* cdf of exponential distribution */
{return 1.-exp(-x);}





#define CLAS 500 /*sets number of classes for the chi-square test*/

static long int obs[CLAS];

int main()
/* demonstrates the use of the algorithm
   and conducts a chi-square test */
{ int i;
 double x;

 setseed(13324821); /*set seed for uniform generator*/

 printf("Demo for Algorithm 7.1\nfirst we display ten numbers generated from my_pdf()\n");
 setup07_01(my_cdf,my_pdf,1.e-8,20,2000,0.,100.);/* calls set-up*/
 for(i=0;i<10;i++){
   x = generator07_01();
   printf("x %f\n",x);
 }
 free07_01();/* to free the memory */

 printf("\nNow we make a chi-square test with sample size %d for the normal distribution\n\n", CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;
 setup07_01(cdf_normal,pdf_normal,1.e-8,20,2000,-10.,10.);/* calls set-up*/
 for(i=0;i<CLAS*CLAS;i++){
   x = generator07_01();
   obs[(int)(cdf_normal(x)*CLAS)]++;
 }
 free07_01();/* to free the memory */

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}

