/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 9.5 (Decreasing Hazard Rate dynamic Thinning), p. 198 
 *
 * this file: demo_algo09_05.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo09_05 demo_algo09_05.c algo09_05.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"


/* Inverse cumulative hazard rate of the distribution
   you can insert your own inverse cumulative hazard rate here
*/
double my_hazard(double x)
/* hazard rate of the pareto distribution with a = 2. */
{ return 2./(x+1.);
}




#define CLAS 500 /*sets number of classes for the chi-square test*/

static long int obs[CLAS];

int main()
/* demonstrates the use of the algorithm
   and conducts a chi-square test */
{ int i;
 double x;

 setseed(133248); /*set seed for uniform generator*/

 printf("Demo for Algorithm 9.5\nfirst we display ten numbers generated from my_hazard()\n\n");
 for(i=0;i<10;i++){
   x = generator09_05(my_hazard);
   printf("x %f\n",x);
 }
 
 printf("\nNow we make a chi-square test with sample size %d \n\n",CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;

 for(i=0;i<CLAS*CLAS;i++){
   x = generator09_05(my_hazard);
   obs[(int)(cdf_pareto(x,2.)*CLAS)]++;
 }

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}


