/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 10.9 (Discrete-Hazard-Rate-Inversion), p. 235
 *
 * this file: demo_algo10_09.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo10_09 demo_algo10_09.c algo10_09.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "uniform.h"
#include "statutil.h"

#define A 0.85

double my_hazard(int i)
{ return A/sqrt(i+1.);
}



#define CLAS 150 /*set numberq of integers for which frequency is counted*/
#define N 100000 /* sample size */

static long int obs[CLAS];

int main()
/* demonstrates the use of the algorithm
   and compares probabilities to observed relative frequencies */
{ int i,k;
  double p,cdf;

 setseed(133248); /*set seed for uniform generator*/


 printf("Demo for Algorithm 10.9\nfirst we display ten numbers generated from my_hazard()\n");
 for(i=0;i<10;i++){
   k =  generator10_09(my_hazard);/* calls the sampling*/ 
   printf("k %d\n",k);
 }
 
 printf("\nNow we count the frequencies with sample size %d\n\n", N);
 for(i=0;i<CLAS;i++) obs[i]=0;

 for(i=0;i<N;i++){
   k =  generator10_09(my_hazard);/* calls the sampling*/ 
   if(k<CLAS) obs[k]++;
 }


 printf("  k   CDF(k)  Probability(k)   Observed relative frequency    ratio\n");
 cdf = 0.;
 for(i=0;i<CLAS&&(i<10||p>0.1/N);i++){
   if(i==0) p = my_hazard(0);
   else p=p*my_hazard(i)*(1.-my_hazard(i-1))/my_hazard(i-1);
   cdf+=p;
   if(obs[i]>0)
   printf("%2d   %f       %f         %f       %f\n",i,cdf,p,obs[i]/(double)N,p/(obs[i]/(double)N));
 }

 printf("\n");

 return 0;
}

