/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 14.6 (Perfect Random Walk Metropolis Sampler, p. 382)
 *
 * this file: demo_algo14_06.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo14_06 demo_algo14_06.c algo14_06.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"


/* PDF of the distribution and its derivative (dpdf)
   you can insert your own distribution here
*/

double my_pdf(double x)
{ if(x>=2.&&x<=4.) return 4.-x;
  return 0.;}
/* triangular on (2,4) */



#define CLAS 200 /*sets number of classes for the chi-square test*/

static long int obs[CLAS];

int main()
/* demonstrates the use of the algorithm
   and conducts a chi-square test */
{ int i;
 double x;

 setseed(13324821); /*set seed for uniform generator*/ 

 printf("\n\nDemo for Algorithm 14.6\nfirst we display ten numbers generated from my_pdf()\n");
 for(i=0;i<10;i++){
   x = generator14_06(my_pdf,2.,4.,0.5);
   printf("x %f\n",x);
 }

 printf("\nChi-square test with sample size %d for the half-normal distribution\n\n", CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;
 for(i=0;i<CLAS*CLAS;i++){
   x = generator14_06(pdf_halfnorm,0.,6.,1.5);
   obs[(int)(cdf_halfnorm(x)*CLAS)]++;
 }

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}

