#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "statutil.h"

#ifndef M_PI
#define M_PI 3.141592653589793238
#endif

#define MIN(a,b) ((a)<(b)?a:b)


/***************************************************************/
double pdf_tri(double x)
/* PDF for the triangular distribution on (0,1) */
{ return 2.*x;
}
double cdf_tri(double x)
/* CDF for the triangular distribution on (0,1) */
{ return x*x;
}
double pdf_tri2(double x)
/* PDF for a second triangular distribution on (0,1) */
{ return 2.-2.*x;
}
double cdf_tri2(double x)
/* CDF for the second triangular distribution on (0,1) */
{ return x*(2.-x);
}

double pdf_oox1_3(double x)
/* density 1/x on (1,3) */
{ 
  if( x < 1. || x > 3.) return 0.;
  return 1./x;
}
double dpdf_oox1_3(double x)
/* derivative of the density 1/x on (1,3) */
{ 
  if( x < 1. || x > 3.) return 0.;
  return -1./(x*x);
}
double cdf_oox1_3(double x)
/* CDF for density 1/x on (1,3) */
{ return log(x)/log(3.);
}

double pdf_halfnorm(double x)
/* CDF of the standard normal distribution restricted to positive values*/
{ if(x>=0.) return 2.*pdf_normal(x);
  return 0.;
}
double cdf_halfnorm(double x)
/* CDF of the standard normal distribution restricted to positive values*/
{ return 2.*(cdf_normal(x)-0.5);
}

double cdf_cauchy(double x)
/* CDF of Cauchy distribution */
{ return 0.5+atan(x)/M_PI;
}

double pdf_gamma3(double x)
/* PDF of gamma distribution with shape parameter alpha = 3 */
{ 
  return (x<=0. ? 0. : exp(-x)*x*x);
}

double cdf_gamma3(double x)
/* CDF of Gamma distribution with shape parameter alpha=3 */
{ return (2.-exp(-x)*(x*x+2.*x+2.))*0.5;
}

double pdf_normal(double x)
/* PDF of standard normal distribution */
#define OOSQRT2PI 0.3989422804014330
{ return(OOSQRT2PI*exp(-x*x*0.5));
}
double dpdf_normal(double x)
/* derivative of the PDF of standard normal distribution */
{ return(-x*OOSQRT2PI*exp(-x*x*0.5));
}
double ddpdf_normal(double x)
/* second derivative of the PDF of standard normal distribution */
{ return((x*x-1.)*OOSQRT2PI*exp(-x*x*0.5));
}
#undef OOSQRT2PI

double cdf_weibull(double x, double alpha, double beta)
     /* cdf of the Weibull distribution */
{ return 1.-exp(-alpha*exp(log(x)*beta));
}

double cdf_pareto (double x, double a)
{ return 1.-exp(-a*log(1.+x));
}


/*******************************************************************/
/*par[0]..R, par[1]..S1, par[2]..S2*/

double hn(double x[2],double par[]) /*log of bivariate normal density*/
{return
(-0.5/(1.-par[0]*par[0])*(x[0]*x[0]*(1./(par[1]*par[1]))-
    2.*par[0]*x[0]*x[1]*(1./(par[1]*par[2]))+x[1]*x[1]*(1./(par[2]*par[2]))));}

double hnx(double x[2],double par[]) /*Partielle Ableitung hn nach x*/
{return(-0.5/(1.-par[0]*par[0])*
	(2*x[0]*(1./(par[1]*par[1]))-2.*par[0]*x[1]*(1./(par[1]*par[2]))));}

double hny(double x[2],double par[]) /*Partielle Ableitung hn nach y*/
{ return(-0.5/(1.-par[0]*par[0])*
	 (-2.*par[0]*x[0]*(1./(par[1]*par[2]))+2*x[1]*(1./(par[2]*par[2]))));}

/*******************************************************************/
/****************************************************************/
/* Verteilungsfunktion von N(0,1) programmiert nach             */
/* Kennedy/Gentle "Statistical Computing" S 90 - 92.            */
/****************************************************************/
extern double cdf_normal(double x)
/* CDF of standard normal distribution */
{
  int  help;
  double xx,x2,x3,x4,x5,x6,x7,x8,zahler,nenner;

  static double
        ep0=242.66795523053175,ep1=21.979261618294152,
        ep2=6.9963834886191355,ep3= -3.5609843701815385e-2,
        eq0=215.0588758698612,eq1=91.164905404514901,
        eq2=15.082797630407787,/*eq3=1,*/
        zp0=300.4592610201616005,zq0=300.4592609569832933,
        zp1=451.9189537118729422,zq1=790.9509253278980272,
        zp2=339.3208167343436870,zq2=931.3540948506096211,
        zp3=152.9892850469404039,zq3=638.9802644656311665,
        zp4=43.16222722205673530,zq4=277.5854447439876434,
        zp5=7.211758250883093659,zq5=77.00015293522947295,
        zp6=0.5641955174789739711,zq6=12.78272731962942351,
        zp7= -1.368648573827167067e-7,/*zq7=1,*/
        dp0= -2.99610707703542174e-3,dq0=1.06209230528467918e-2,
        dp1= -4.94730910623250734e-2,dq1=1.91308926107829841e-1,
        dp2= -2.26956593539686930e-1,dq2=1.05167510706793207,
        dp3= -2.78661308609647788e-1,dq3=1.98733201817135256,
        dp4= -2.23192459734184686e-2/*,dq4=1*/;


  if(x<0)
    help=0;
  else
    help=1;
  x=fabs(x)/sqrt(2.);
  if(x<0.5)
    {
     x2=x*x;
     x4=x2*x2;
     x6=x4*x2;
     zahler=ep0+ep1*x2+ep2*x4+ep3*x6;
     nenner=eq0+eq1*x2+eq2*x4+x6;
     if(help)
       return(0.5*(1+x*zahler/nenner));
     else
       return(0.5*(1-x*zahler/nenner));
    }
  else if(x<4)
    {
     x2=x*x;
     x3=x2*x;
     x4=x3*x;
     x5=x4*x;
     x6=x5*x;
     x7=x6*x;
     zahler=zp0+zp1*x+zp2*x2+zp3*x3+zp4*x4+zp5*x5+zp6*x6+zp7*x7;
     nenner=zq0+zq1*x+zq2*x2+zq3*x3+zq4*x4+zq5*x5+zq6*x6+x7;

     if(help)
       return(0.5*(2-exp(-x2)*zahler/nenner));
     else
       return(0.5*exp(-x2)*zahler/nenner);
    }
  else if(x<50)
    {
     xx=x*x;
     x2=1/xx;
     x4=x2*x2;
     x6=x4*x2;
     x8=x6*x2;
     zahler=dp0+dp1*x2+dp2*x4+dp3*x6+dp4*x8;
     nenner=dq0+dq1*x2+dq2*x4+dq3*x6+x8;
     if(help)
       return(1-0.5*(exp(-xx)/x)*
             (1/sqrt(M_PI)+zahler/(xx*nenner)));
     else
       return(0.5*(exp(-xx)/x)*(1/sqrt(M_PI)+zahler/(xx*nenner)));
    }
  else if(help)
         return(1.);
       else
         return(0.);
}
/****************************************************************
 * cumulated distribution function - CHI^2 distribution         *
 * for n degrees of freedom,                                    *
 * Approximation from Wilson & Hilferty                         *
 * Kendall Stuart I, p 371                                      *
 ****************************************************************/
double cdf_chia(double x,int n)             /* approximation */
{
  double chix,y;

  y=2./(9.*n);
  chix=(exp(log(x/n)/3)-1+y)/sqrt(y);
  return(cdf_normal(chix));
}
/****************************************/

/*Chi-2 test for uniform distribution*/ 

double chi2test(long b[]/*obsereved frequencies*/, 
                long l,/*number of classes*/
		long wid,/*sample size*/ 
		int printflag/*0..no output, 1.. little, 2..more*/)
{  long int i;
   double chi2=0.,erw,pval;

   erw=(double)wid/l;
   if (erw<5.) printf("Error chi2test: expected frequency smaller than 5\n");
   for (i=0;i<l;++i)
     {
       if (printflag==2) printf("%ld:%ld;",i,b[i]);
       chi2+= (b[i]-erw)*(b[i]-erw);
     }
   chi2/=erw;
   pval=1.-cdf_chia(chi2,l-1);
   if (printflag>=1)
     {
       printf("Chi2-test: samplesize=%ld  number of classes= %ld \n",wid,l);
       printf("Chi2-value %f   Approximate P-Value:  %f\n",chi2,pval);
       if(pval>0.05) printf("Chi2-Test PASSED\n");
       else if(pval>0.0001){ 
	 printf("Chi2-Test not passed\n");
         printf("repeat test with a different seed for the uniform generator and a larger samplesize!\n");
       }
       else printf("Chi2-Test NOT PASSED!!! there seems to be a problem with the pdf, cdf or the generator!!\n");
     }


   return(chi2);
}

/*******************************************************************/

void printmatrix(int d, double M[])
     /* prints a square matrix that is stored in a vector of length (d+1)^2 */
{ int i,j;

 printf("\n");
 for(i=1;i<=d;i++){
   for(j=1;j<=d;j++) 
     printf("%6.3f ",M[(d+1)*i+j]);
   printf("\n");
 }
 printf("\n");
}
/*******************************************************************/
/*computes log(k!) using the Stirling approximation and a table for
  small values of k*/
double lnkfakt(int k)
     /* evaluates log(k!) */
{ static double lnw2p=0.9189385332046727418 ,tabl[10]=
    {0., 0., 0.69314718055994530942, 1.7917594692280550008, 
       3.1780538303479456196,4.7874917427820459942, 6.5792512120101009951, 
       8.5251613610654143, 10.604602902745250228, 12.801827480081469611};
  double dk,edk;
  if (k<=9) return(tabl[k]);
  dk=k; 
  edk=1./dk; 
  return(lnw2p+(dk+0.5)*log(dk)-dk+edk*(1./12.-edk*edk* (1./360.)));
}
/**************************************************************************/
/***************************************************************/
/*utility for computing mean, variance and correlation ***********/

/*correlation for parent population and not for sample 
  (sums divided through n not through n-1  !!!!!!!!!*/

MEANCOR *init_meancor()
{ 
  MEANCOR *p;
  p = malloc(sizeof(MEANCOR)); 
  p->mx = 0.;
  p->sx = 0.;
  p->my = 0.;
  p->sy = 0.;
  p->sxy = 0.;
  p->i = 0;
  p->cookie = -9999;
  return p;
}

void free_meancor(MEANCOR *p)
{ free(p);
}

void update_meancor(MEANCOR *p, double xi, double yi)
{ double dx,dy;
  if(p==NULL || p->cookie != -9999){
    printf("error update_meancor; cookie check failed!! exiting\n");
    exit(1);
  }
  p->i++;
  dx= (xi - p->mx)/(double)p->i;
  p->mx+=dx;
  p->sx+= p->i*(p->i-1.)*dx*dx;
  dy= (yi - p->my)/(double)p->i;
  p->my+=dy;
  p->sy+= p->i*(p->i-1.)*dy*dy;
  p->sxy+= p->i*(p->i-1.)*dx*dy;
}

void printresult_meancor(MEANCOR *p)
{
  if(p==NULL || p->cookie != -9999){
    printf("error printresult_meancor; cookie check failed!! exiting\n");
    exit(1);
  }
  printf("correlation: %f \n",p->sxy/sqrt(p->sx*p->sy));
}

